@@ -13,7 +13,6 @@ use crate::secrets::*;
 use crate::Reference;
 
 use crate::token_cache::{RegistryOperation, RegistryToken, RegistryTokenType, TokenCache};
-use crate::www_authenticate::{Challenge, ChallengeFields, RawChallenge, WwwAuthenticate};
 use anyhow::{anyhow, Context};
 use futures_util::future;
 use futures_util::stream::StreamExt;
@@ -25,6 +24,7 @@ use std::collections::HashMap;
 use std::convert::TryFrom;
 use tokio::io::{AsyncWrite, AsyncWriteExt};
 use tracing::{debug, trace, warn};
+use www_authenticate::{Challenge, ChallengeFields, RawChallenge, WwwAuthenticate};
 
 const MIME_TYPES_DISTRIBUTION_MANIFEST: &[&str] = &[
     "application/vnd.docker.distribution.manifest.v2+json",
@@ -134,10 +134,7 @@ impl TryFrom<ClientConfig> for Client {
             #[cfg(feature = "native-tls")]
             () => client_builder.danger_accept_invalid_hostnames(config.accept_invalid_hostnames),
             #[cfg(not(feature = "native-tls"))]
-            () => {
-                warn!("Cannot change value of `accept_invalid_hostnames`: missing 'native-tls' feature");
-                client_builder
-            }
+            () => client_builder,
         };
 
         for c in &config.extra_root_certificates {
@@ -985,6 +982,7 @@ pub struct ClientConfig {
     pub protocol: ClientProtocol,
 
     /// Accept invalid hostname. Defaults to false
+    #[cfg(feature = "native-tls")]
     pub accept_invalid_hostnames: bool,
 
     /// Accept invalid certificates. Defaults to false
@@ -1116,6 +1114,7 @@ mod test {
         HELLO_IMAGE_TAG_AND_DIGEST,
     ];
     const DOCKER_IO_IMAGE: &str = "docker.io/library/hello-world:latest";
+    const GHCR_IO_IMAGE: &str = "ghcr.io/krustlet/oci-distribution/hello-wasm:v1";
 
     #[test]
     fn test_apply_accept() -> Result<(), anyhow::Error> {
@@ -1892,4 +1891,15 @@ mod test {
             "unsupported media type: application/vnd.docker.distribution.manifest.list.v2+json"
         );
     }
+
+    #[tokio::test]
+    async fn test_pull_ghcr_io() {
+        let reference = Reference::try_from(GHCR_IO_IMAGE).expect("failed to parse reference");
+        let mut c = Client::default();
+        let (manifest, _manifest_str) = c
+            .pull_manifest(&reference, &RegistryAuth::Anonymous)
+            .await
+            .unwrap();
+        assert_eq!(manifest.config.media_type, manifest::WASM_CONFIG_MEDIA_TYPE);
+    }
 }
